/*
 * buttons.c
 *
 * Created: 10.04.2015 19:18:30
 *  Author: vebjornr
 */ 

#include "buttons.h"
#include "definitions.h"
#include "adc.h"
#include "can.h"
#include <avr/io.h>

void buttons_send_on_can(buttons_t *buttons){
	
	CAN_packet can_msg = {STEERING_WHEEL_ID, 4, {}};
		
	can_msg.data[0] |= (buttons->blink_left << 7);
	can_msg.data[0] |= (buttons->blink_right << 6);
	can_msg.data[0] |= (buttons->dead_mans_1 << 5);
	can_msg.data[0] |= (buttons->dead_mans_2 << 4);
	can_msg.data[0] |= (buttons->horn << 3);
	can_msg.data[0] |= (buttons->joy_sw << 2);
	can_msg.data[0] |= (buttons->lights_far << 1);
	can_msg.data[0] |= (buttons->lights_near << 0);
	
	can_msg.data[1] |= (buttons->phone << 7);
	can_msg.data[1] |= (buttons->wiper << 6);
	
	can_msg.data[2] = buttons->joy_y;
	can_msg.data[3] = (buttons->joy_y >> 8);
	
	can_msg.data[4] = buttons->volume;
	can_msg.data[5] = (buttons->volume >> 8);
	
	can_msg.data[6] = buttons->fans;
	can_msg.data[7] = (buttons->fans >> 8);
	
	//try to send on all MOBs
	for (int i=0; i<14; i++)
	{
		if (can_tx(i, &can_msg))
		{
			break;
		}
	}
	
}

void buttons_read(buttons_t *buttons){
	
	buttons->blink_left = get_bit(PB0, PINB);
	buttons->blink_right = get_bit(PB3, PINB);
	buttons->dead_mans_1 = get_bit(PE3, PINE);
	buttons->dead_mans_2 = get_bit(PD2, PIND); // extra
	buttons->horn = get_bit(PD0, PIND);
	buttons->joy_sw = get_bit(PD1, PIND);
	buttons->lights_far = get_bit(PB2, PINB);
	buttons->lights_near = get_bit(PB1, PINB);
	buttons->phone = get_bit(PE4, PINE);
	buttons->wiper = get_bit(PE5, PINE);

	buttons->volume = adc_read(VOLUME);
	buttons->fans = adc_read(FANS);
	buttons->joy_x = adc_read(JOY_X);
	buttons->joy_y = adc_read(JOY_Y);
	
}

void buttons_init(){
	
		//set as input
		clear_bit(DDRB, PB0);
		clear_bit(DDRB, PB3);
		clear_bit(DDRB, PB2);
		clear_bit(DDRB, PB1);
		
		clear_bit(DDRE, PE3);
		clear_bit(DDRE, PE4);
		clear_bit(DDRE, PE5);
		
		clear_bit(DDRD, PD2);
		clear_bit(DDRD, PD0);
		clear_bit(DDRD, PD1);
		
		// set pullups
		set_bit(PORTB, PB0);
		set_bit(PORTB, PB3);
		set_bit(PORTB, PB2);
		set_bit(PORTB, PB1);
		
		set_bit(PORTE, PE3);
		set_bit(PORTE, PE4);
		set_bit(PORTE, PE5);
		
		set_bit(PORTD, PD2);
		set_bit(PORTD, PD0);
		set_bit(PORTD, PD1);
		
}
