/*
 * lcd.c
 *
 * Created: 17.04.2015 14:42:07
 *  Author: vebjornr
 */ 

#include "lcd.h"
#include "USART.h"
#include "definitions.h"
#include "timer.h"
#include <util/delay.h>
#include <stdio.h>

void lcd_init(){
	
		_delay_ms(200);
		usart1_puts("<control>r");
		_delay_ms(1000);
		
		//lcd_set_splash_screen();
		//_delay_ms(1000);
		
		lcd_clear_all_lines();
		
		//lcd_set_brightness(29); // max
		
		//write display text
		usart1_putc(254);	//move cursor
		usart1_putc(LINE1);	//first line
		usart1_puts("SPEED:    kmh  F:   ");
		usart1_putc(254);
		usart1_putc(LINE2);
		usart1_puts("TIME:");
		usart1_putc(254);
		usart1_putc(LINE3);
		usart1_puts("TORQUE:");
		usart1_putc(254);
		usart1_putc(LINE4);
		usart1_puts("STATUS:        V:   ");
	
}

void lcd_set_brightness(uint8_t brightness){
	
	if (brightness < 0 || brightness > 29) {return;}

	usart1_putc(124); // control char
	usart1_putc(128 + brightness);
	
	
}

void lcd_clear_all_lines(void){
	
	//clear LCD Display
	usart1_putc(254);	//move cursor
	usart1_putc(LINE1);	//first line
	usart1_puts("                    ");
	
	usart1_putc(254);	//move cursor
	usart1_putc(LINE2);	//second line
	usart1_puts("                    ");
	
	usart1_putc(254);	//move cursor
	usart1_putc(LINE3);	//second line
	usart1_puts("                    ");
	
	usart1_putc(254);	//move cursor
	usart1_putc(LINE4);	//second line
	usart1_puts("                    ");
	
}

void lcd_goto_line_place(uint8_t line, uint8_t place){
	
	usart1_putc(254);
	usart1_putc(line + place);
	
}

void lcd_write_line_place(char *text, uint8_t line, uint8_t place, uint8_t max_space){
	
	lcd_goto_line_place(line, place);
	for (int i=0; i<max_space; i++)
	{
		usart1_putc(' '); // clear previous text
	}
	lcd_goto_line_place(line, place);
	usart1_puts(text);
	
}

void lcd_write(output_t type, int16_t value){
	
	char out[10] = {};
	
	switch(type){
		
		case SPEED:{
			
			int print_speed = (int)((((double)value)*3.6)/100.0);
			if (print_speed >= 0 && print_speed <= 99)
			{
				sprintf(out, "%d kmh", print_speed); // convert to km/h
				//sprintf(out, "%d kmh", value);
				lcd_write_line_place(out, LINE1, 7, 7);
			} else {
				sprintf(out, "?? kmh"); // convert to km/h
				//sprintf(out, "%d kmh", value);
				lcd_write_line_place(out, LINE1, 7, 7);
			}
			break;	
		}
		case TIME:{
			if(value >=0){
				int8_t minutes = value/60;
				int8_t seconds = value - (60*minutes);
				sprintf(out, " %02d:%02d", minutes, seconds);
				lcd_write_line_place(out, LINE2, 5, 7);
			}else{
				int8_t minutes = (-value)/60;
				int8_t seconds = (-value) - (60*minutes);
				sprintf(out, "-%02d:%02d", minutes, seconds);
				lcd_write_line_place(out, LINE2, 5, 7);
			}
			break;	
		}
		case TORQUE:{
			int adjusted_torque = value - 511;
			sprintf(out, "%d", adjusted_torque);
			lcd_write_line_place(out, LINE3, 8, 4);
			break;	
		}
		case VOLUME_OUT:{
			uint16_t volume_percentage = (value*100)/255;
			sprintf(out, "%d", volume_percentage);
			lcd_write_line_place(out, LINE4, 17, 3);
			break;
		}
		case FANS_OUT:{
			uint16_t fans_percentage = (value*100)/255;
			sprintf(out, "%d", fans_percentage);
			lcd_write_line_place(out, LINE1, 17, 3);
			break;
		}
		case STATUS:{
			
			break;
		}
		default:{
			//this state should never be reached. Print question-marks to indicate that something is really wrong
			lcd_write_line_place("???", LINE4, 8, 3);
			break;
		}		
	}
	
	
}


void lcd_set_splash_screen(void){
	
	//This function is only required to be called once on the display. 
	//The splash screen is stored in the screens persistent memory
	
	
	lcd_clear_all_lines();
	lcd_write_line_place(SPLASH_LINE_1, LINE1, 0, 20);
	lcd_write_line_place(SPLASH_LINE_2, LINE2, 0, 20);
	usart1_putc(124);
	usart1_putc(10);
	
}