/*
 * main.c
 *
 * Created: 09.04.2015 17:20:49
 *  Author: vebjornr
 */ 


#include "definitions.h"
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <stdio.h>

#include "can.h"
#include "USART.h"
#include "adc.h"
#include "buttons.h"
#include "lcd.h"
#include "timer.h"

#define RACE_TIME 39 //minutes
volatile int clk_ticks = 0;
volatile int seconds_remaining = RACE_TIME*60;
volatile buttons_t buttons;
uint16_t speed = 0;
volatile uint8_t update_screen = 0;

void can_read_buttons(CAN_packet *p, unsigned char mob);
void can_read_speed(CAN_packet *p, unsigned char mob);


int main(void)
{
	usart1_init(MYUBRR);	// set baud rate to Display to 9600
	timer_init();			//enable and start timer0
	lcd_init();				// clear and prepare lcd display
	can_init();				//init CAN-controller
	
	sei();					// enable global interrupts
	
	prepare_rx(1, 0x01, 0x7FF, can_read_buttons);	// start listening for CAN-packets with ID 0x01
	prepare_rx(2, 0x0A, 0x7FF, can_read_speed);		// start listening for CAN-packets with ID 0xA0
	
	
    while(1)
    {
		
		if (update_screen)
		{
			if(buttons.reset_time == 0){seconds_remaining = RACE_TIME*60;}	// reset the timer, if the driver has pushed the reset button
			lcd_write(SPEED, speed);										// write speed (km/h) to display
			lcd_write(TIME, seconds_remaining);								// write time remaining to display
			lcd_write(TORQUE, buttons.joy_y);								// write torque reference to display
			lcd_write(VOLUME_OUT, buttons.volume>>2);						// write headset volume to display
			lcd_write(FANS_OUT, buttons.fans>>2);							// write fan-intensity to display
			update_screen = 0;
		}
		
    }
}



void can_read_buttons(CAN_packet *p, unsigned char mob){

	

	buttons.blink_left =	(p->data[0] & (1<<7)) >> 7;
	buttons.blink_right =	(p->data[0] & (1<<6)) >> 6;
	buttons.dead_mans_1 =	(p->data[0] & (1<<5)) >> 5;
	buttons.dead_mans_2 =	(p->data[0] & (1<<4)) >> 4;
	buttons.horn =			(p->data[0] & (1<<3)) >> 3;
	buttons.joy_sw =		(p->data[0] & (1<<2)) >> 2;
	buttons.lights_far =	(p->data[0] & (1<<1)) >> 1;
	buttons.lights_near =	(p->data[0] & (1<<0)) >> 0;
	
	buttons.phone = (p->data[1] & (1<<7)) >> 7;
	buttons.wiper = (p->data[1] & (1<<6)) >> 6;
	buttons.unused = (p->data[1] & (1<<5)) >> 5;
	buttons.reset_time = (p->data[1] & (1<<4)) >> 4;
	buttons.brake = (p->data[1] & (1<<3)) >> 3;
	
	buttons.joy_y = (p->data[2] << 2);
	buttons.joy_y |= (p->data[3]);
	
	buttons.volume = p->data[4] << 2;
	buttons.volume |= (p->data[5]);
	
	buttons.fans = p->data[6] << 2;
	buttons.fans |= (p->data[7]);
	
	
	update_screen = 1;
	//prepare_rx(mob, 0x01, 0x7FF, can_rx_complete);	
	
}

void can_read_speed(CAN_packet *p, unsigned char mob){
	
	speed = p->data[0] << 8;
	speed |= p->data[1];
	update_screen = 1;
	
}

ISR(USART0_RX_vect)
{
	// Code to be executed when the USART receives a byte here
	//uint8_t b = UDR0;
	//usart0_putc(b);
	
}