#!/usr/bin/python3

import socket
import sys
import os
import wiringpi2 as wiringpi
import serial
import _thread as thread
from time import sleep
import time

server_address = '/tmp/uds_dnvff'

class Buttons:
    blink_left = 0
    blink_right = 0
    dead_mans1 = 0
    dead_mans2 = 0
    horn = 0
    joy_sw = 0
    lights_far = 0
    lights_near = 0
    phone = 0
    wiper = 0
    joy_y = 0

class Server:
 
    def __init__(self, address):
        print('Opening serial port')
        self.last_transmission = time.clock()
        self.counter = 0
        self.buttons = Buttons()
        self.port = serial.Serial("/dev/ttyAMA0", baudrate=38400, bytesize=serial.EIGHTBITS, parity=serial.PARITY_NONE, stopbits=serial.STOPBITS_ONE)

        # Make sure the socket does not already exist
        try:
            os.unlink(address)
        except OSError:
            if os.path.exists(address):
                raise
		
        # Create a UDS (UDP) socket
        self.sock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)

        # Bind the socket to the port
        print ('starting up on %s' % server_address)
        self.sock.bind(server_address)

        print('Setting file permissions..')
        os.chmod(address, 0o777)

        # Listen for incoming connections
        self.sock.listen(1)
        print('Listening on socket..')
        
        self.port.write('Server started\n'.encode('utf-8'))
        
        
        
    def listen_on_TCP(self):

        while True:
            # Wait for a connection
            print ('waiting for a connection')
            self.connection, self.client_address = self.sock.accept()
            self.connected_state = True
            try:
                print ('connection')

                # Receive the data in small chunks and retransmit it
                while True:
                    data = self.connection.recv(32)
                    print('received "%s"' % data)
                    if data:
                        print('got data. passing it on. (%s)' % data)
                        #connection.send(data)
                        self.port.write(data)
                    else:
                        print('no more data. Client closed connection')
                        break
                    
            finally:
                # Clean up the connection
                print('closing connection')
                self.connected_state = False
                self.connection.close()
     
    def parse_data(self, data):
        if data[0] == 254 and self.counter == 0:
            self.counter = self.counter + 1
            print('start of message')
        #else:
        #    self.counter = 0
        if data[0] == 1 and self.counter == 1 : #buttons data
            self.counter = self.counter + 1
            print('ID 1')
        #else:
        #    self.counter = 0
        if data[0] == 254 and self.counter == 2:
            self.counter = self.counter + 1
            print('so far so good..')
            return
        #else:
        #    self.counter = 0
            
        if self.counter == 3:
            print('reading buttons!')
            self.buttons.blink_left = (data[0] & (1<<7)) >> 7
            self.buttons.blink_right = (data[0] & (1<<6)) >> 6
            self.buttons.dead_mans1 = (data[0] & (1<<5)) >> 5
            self.buttons.dead_mans2 = (data[0] & (1<<4)) >> 4
            self.buttons.horn = (data[0] & (1<<3)) >> 3
            self.buttons.joy_sw = (data[0] & (1<<2)) >> 2
            self.buttons.lights_far = (data[0] & (1<<1)) >> 1
            self.buttons.lights_near = (data[0] & (1<<0)) >> 0
            self.counter = self.counter + 1
            return
        if self.counter == 4:
            print('reading more buttons!')
            self.buttons.phone = (data[0] & (1<<7)) >> 7
            self.buttons.wiper = (data[0] & (1<<6)) >> 6
            self.counter = self.counter + 1
            return
        if self.counter == 5:
            print('reading joy_y 1')
            self.buttons.joy_y = (data[0] << 2)
            self.counter = self.counter + 1
            return
        if self.counter == 6:
            print('reading joy y 2')
            self.buttons.joy_y = self.buttons.joy_y | (data[0])
            self.counter = self.counter + 1
            return
        if self.counter == 7:
            print('joy_y: %d' % (self.buttons.joy_y))
            self.counter = 0
        print('counter: %d' % (self.counter))  
        
    def listen_on_UART(self):
        print('listening on uart...')

        while True:
            data = self.port.readline()
            #self.parse_data(data)
            #print(data[0])
            #data = bytes([ord(data) & 0x7F])
            print(data)
            if self.connected_state == True and time.clock() > self.last_transmission + 1:
                self.connection.send(data)
                self.last_transmission = time.clock()
            else:
                print('not connected')
                
    def run(self):
        self.connected_state = False
        try:
            thread.start_new_thread( self.listen_on_TCP, () )
            thread.start_new_thread( self.listen_on_UART, () )
        except:
            print('Error: unable to start threads')
            sys.exit(1)
        
        #sleep forever, while threads run
        while 1:
            sleep(10)    
            
server = Server(server_address)
server.run()

